@@ -23,6 +23,7 @@
 
 
 #include "CaretDataFile.h"
+#include "SceneFileBasePathTypeEnum.h"
 
 namespace caret {
 
@@ -54,6 +55,8 @@ namespace caret {
 
         void addScene(Scene* scene);
         
+        int32_t getIndexOfScene(const Scene* scene) const;
+        
         void insertScene(Scene* newScene,
                          const Scene* insertAboveThisScene);
         
@@ -62,10 +65,13 @@ namespace caret {
         
         int32_t getNumberOfScenes() const;
         
-        Scene* getSceneAtIndex(const int32_t indx);
+        Scene* getSceneAtIndex(const int32_t indx) const;
         
         Scene* getSceneWithName(const AString& sceneName);
         
+        void moveScene(Scene* scene,
+                       const int32_t indexDelta);
+        
         void removeScene(Scene* scene);
         
         void removeSceneAtIndex(const int32_t indx);
@@ -78,8 +84,48 @@ namespace caret {
         
         const GiftiMetaData* getFileMetaData() const;
         
+        AString getBalsaStudyID() const;
+        
+        void setBalsaStudyID(const AString& balsaStudyID);
+        
+        AString getBalsaStudyTitle() const;
+        
+        void setBalsaStudyTitle(const AString& balsaStudyTitle);
+        
+        SceneFileBasePathTypeEnum::Enum getBasePathType() const;
+        
+        void setBasePathType(const SceneFileBasePathTypeEnum::Enum basePathType);
+        
+        AString getBalsaCustomBaseDirectory() const;
+        
+        void setBalsaCustomBaseDirectory(const AString& balsaBaseDirectory);
+        
+        bool findBaseDirectoryForDataFiles(AString& baseDirectoryOut,
+                                           std::vector<AString>& missingFileNamesOut,
+                                           AString& errorMessageOut) const;
+        
+        std::vector<AString> getBaseDirectoryHierarchyForDataFiles(const int32_t maximumAncestorCount = 25);
+        
+        std::vector<AString> getAllDataFileNamesFromAllScenes() const;
+        
         void reorderScenes(std::vector<Scene*>& orderedScenes);
+        
+        int32_t getSceneIndexFromNumberOrName(const AString& numberOrName);
+        
+        Scene* releaseScene(const int32_t& index);
 
+        AString getDefaultZipFileName() const;
+        
+        AString getBalsaExtractToDirectoryName() const;
+        
+        void setBalsaExtractToDirectoryName(const AString& balsaExtractToDirectoryName);
+
+        AString getDefaultExtractToDirectoryName() const;
+        
+        virtual bool isModified() const override;
+        
+        virtual void clearModified() override;
+        
         // ADD_NEW_METHODS_HERE
 
         /** Version of file */
@@ -104,6 +150,21 @@ namespace caret {
         /** the metadata */
         GiftiMetaData* m_metadata;
 
+        /** the BALSA Study ID */
+        AString m_balsaStudyID;
+        
+        /** the BALSA Study Title */
+        AString m_balsaStudyTitle;
+        
+        /** the Base Path Type */
+        SceneFileBasePathTypeEnum::Enum m_basePathType;
+        
+        /** the Custom Base Directory */
+        AString m_balsaCustomBaseDirectory;
+        
+        /** The "extract to" directory name */
+        AString m_balsaExtractToDirectoryName;
+        
         // ADD_NEW_MEMBERS_HERE
 
         /** Version of this SceneFile */
@@ -114,7 +175,7 @@ namespace caret {
     const AString SceneFile::XML_TAG_SCENE_FILE = "SceneFile";
     const AString SceneFile::XML_ATTRIBUTE_VERSION = "Version";
     const AString SceneFile::XML_TAG_SCENE_INFO_DIRECTORY_TAG = "SceneInfoDirectory";
-    const float SceneFile::s_sceneFileVersion = 2.0;
+    const float SceneFile::s_sceneFileVersion = 3.0;
 #endif // __SCENE_FILE_DECLARE__
 
 } // namespace
