@@ -222,7 +222,7 @@ GiftiDataArray::getDataTypeAppropriateForIntent(const NiftiIntentEnum::Enum inte
    }
    else if ((intent == NiftiIntentEnum::NIFTI_INTENT_RGB_VECTOR) ||
             (intent == NiftiIntentEnum::NIFTI_INTENT_RGBA_VECTOR)) {
-      dataTypeOut = NiftiDataTypeEnum::NIFTI_TYPE_UINT8;
+      dataTypeOut = NiftiDataTypeEnum::NIFTI_TYPE_FLOAT32;
    }
    else if (intent == NiftiIntentEnum::NIFTI_INTENT_SHAPE) {
       dataTypeOut = NiftiDataTypeEnum::NIFTI_TYPE_FLOAT32;
@@ -681,7 +681,7 @@ GiftiDataArray::readFromText(const AString text,
                // Decode the Base64 data using VTK's algorithm
                //
                unsigned char* dataBuffer = new unsigned char[data.size()];
-               //const char* textChars = text.toAscii().constData();
+               //const char* textChars = text.toLatin1().constData();
                const uint64_t numDecoded =
                      Base64::decode((unsigned char*)text.toStdString().c_str(),
                                                 data.size(),
@@ -732,7 +732,7 @@ GiftiDataArray::readFromText(const AString text,
                }
                
                 std::ifstream extBinFile(externalFileNameForReading.toStdString().c_str(),
-                                         std::ifstream::in);
+                                         std::ifstream::in | std::ifstream::binary);
                 if (extBinFile.good() == false) {
                         throw GiftiException("Error opening \""
                                             + externalFileNameForReading
@@ -1723,13 +1723,21 @@ const FastStatistics* GiftiDataArray::getFastStatistics() const
     return m_fastStatistics;
 }
 
-const Histogram* GiftiDataArray::getHistogram() const
+const Histogram* GiftiDataArray::getHistogram(const int32_t numberOfBuckets) const
 {
     if (this->getDataType() == NiftiDataTypeEnum::NIFTI_TYPE_FLOAT32) {
+        bool updateHistogramFlag = false;
         if (m_histogram == NULL) {
-            m_histogram.grabNew(new Histogram(100));
+            m_histogram.grabNew(new Histogram(numberOfBuckets));
+            updateHistogramFlag = true;
+        }
+        else if (numberOfBuckets != m_histogramNumberOfBuckets) {
+            updateHistogramFlag = true;
+        }
+        if (updateHistogramFlag) {
+            m_histogram->update(numberOfBuckets, dataPointerFloat, getTotalNumberOfElements());
+            m_histogramNumberOfBuckets = numberOfBuckets;
         }
-        m_histogram->update(dataPointerFloat, getTotalNumberOfElements());
     }
     return m_histogram;
 }
@@ -1772,23 +1780,43 @@ GiftiDataArray::getDescriptiveStatistics(const float mostPositiveValueInclusive,
     return this->descriptiveStatisticsLimitedValues;
 }
 
-const Histogram* GiftiDataArray::getHistogram(const float mostPositiveValueInclusive,
+const Histogram* GiftiDataArray::getHistogram(const int32_t numberOfBuckets,
+                                              const float mostPositiveValueInclusive,
                                               const float leastPositiveValueInclusive,
                                               const float leastNegativeValueInclusive,
                                               const float mostNegativeValueInclusive,
                                               const bool includeZeroValues) const
 {
     if (this->getDataType() == NiftiDataTypeEnum::NIFTI_TYPE_FLOAT32) {
+        bool updateHistogramFlag = false;
         if (m_histogramLimitedValues == NULL)
         {
-            m_histogramLimitedValues.grabNew(new Histogram(100));
+            m_histogramLimitedValues.grabNew(new Histogram(numberOfBuckets));
+            updateHistogramFlag = true;
+        }
+        else if ((numberOfBuckets != m_histogramLimitedValuesNumberOfBuckets)
+                 || (mostPositiveValueInclusive != m_histogramLimitedValuesMostPositiveValueInclusive)
+                 || (leastPositiveValueInclusive != m_histogramLimitedValuesLeastPositiveValueInclusive)
+                 || (leastNegativeValueInclusive != m_histogramLimitedValuesLeastNegativeValueInclusive)
+                 || (mostNegativeValueInclusive != m_histogramLimitedValuesMostNegativeValueInclusive)
+                 || (includeZeroValues != m_histogramLimitedValuesIncludeZeroValues)) {
+            updateHistogramFlag = true;
+        }
+        if (updateHistogramFlag) {
+            m_histogramLimitedValues->update(numberOfBuckets,
+                                             dataPointerFloat, getTotalNumberOfElements(),
+                                             mostPositiveValueInclusive,
+                                             leastPositiveValueInclusive,
+                                             leastNegativeValueInclusive,
+                                             mostNegativeValueInclusive,
+                                             includeZeroValues);
+            m_histogramLimitedValuesNumberOfBuckets = numberOfBuckets;
+            m_histogramLimitedValuesMostPositiveValueInclusive = mostPositiveValueInclusive;
+            m_histogramLimitedValuesLeastPositiveValueInclusive = leastPositiveValueInclusive;
+            m_histogramLimitedValuesLeastNegativeValueInclusive = leastNegativeValueInclusive;
+            m_histogramLimitedValuesMostNegativeValueInclusive = mostNegativeValueInclusive;
+            m_histogramLimitedValuesIncludeZeroValues = includeZeroValues;
         }
-        m_histogramLimitedValues->update(dataPointerFloat, getTotalNumberOfElements(),
-                                         mostPositiveValueInclusive,
-                                         leastPositiveValueInclusive,
-                                         leastNegativeValueInclusive,
-                                         mostNegativeValueInclusive,
-                                         includeZeroValues);
     }
     return m_histogramLimitedValues;
 }
