@@ -27,6 +27,8 @@
 #include <QTabWidget>
 #include <QTimer>
 
+#include "AnnotationFile.h"
+#include "AnnotationSelectionViewController.h"
 #include "BorderSelectionViewController.h"
 #include "Brain.h"
 #include "BrainBrowserWindow.h"
@@ -36,10 +38,12 @@
 #include "CaretDataFile.h"
 #include "CaretPreferences.h"
 #include "ChartableLineSeriesBrainordinateInterface.h"
+#include "ChartTwoOverlaySetViewController.h"
 #include "ChartableMatrixInterface.h"
 #include "ChartToolBoxViewController.h"
 #include "CiftiConnectivityMatrixViewController.h"
-#include "EventBrowserWindowContentGet.h"
+#include "DeveloperFlagsEnum.h"
+#include "EventBrowserWindowDrawingContent.h"
 #include "EventManager.h"
 #include "EventUserInterfaceUpdate.h"
 #include "FiberOrientationSelectionViewController.h"
@@ -112,19 +116,23 @@ BrainBrowserWindowOrientedToolBox::BrainBrowserWindowOrientedToolBox(const int32
                   + "_"
                   + AString::number(browserWindowIndex));
     
-    m_borderSelectionViewController = NULL;
-    m_chartToolBoxViewController = NULL;
+    m_annotationViewController         = NULL;
+    m_borderSelectionViewController    = NULL;
+    m_chartOverlaySetViewController    = NULL;
+    m_chartToolBoxViewController       = NULL;
     m_connectivityMatrixViewController = NULL;
-    m_fiberOrientationViewController = NULL;
-    m_fociSelectionViewController = NULL;
-    m_imageSelectionViewController = NULL;
-    m_labelSelectionViewController = NULL;
-    m_overlaySetViewController = NULL;
+    m_fiberOrientationViewController   = NULL;
+    m_fociSelectionViewController      = NULL;
+    m_imageSelectionViewController     = NULL;
+    m_labelSelectionViewController     = NULL;
+    m_overlaySetViewController         = NULL;
     m_volumeSurfaceOutlineSetViewController = NULL;
 
     m_tabWidget = new QTabWidget();
     
+    m_annotationTabIndex = -1;
     m_borderTabIndex = -1;
+    m_chartOverlayTabIndex = -1;
     m_chartTabIndex = -1;
     m_connectivityTabIndex = -1;
     m_fiberOrientationTabIndex = -1;
@@ -141,6 +149,13 @@ BrainBrowserWindowOrientedToolBox::BrainBrowserWindowOrientedToolBox(const int32
         m_overlayTabIndex = addToTabWidget(m_overlaySetViewController,
                        "Layers");
     }
+    if (isOverlayToolBox) {
+        m_chartOverlaySetViewController = new ChartTwoOverlaySetViewController(orientation,
+                                                                         browserWindowIndex,
+                                                                         this);
+        m_chartOverlayTabIndex = addToTabWidget(m_chartOverlaySetViewController,
+                                                "Chart Layers");
+    }
     if (isOverlayToolBox) {
         m_chartToolBoxViewController = new ChartToolBoxViewController(orientation,
                                                                       browserWindowIndex,
@@ -154,6 +169,12 @@ BrainBrowserWindowOrientedToolBox::BrainBrowserWindowOrientedToolBox(const int32
         m_connectivityTabIndex = addToTabWidget(m_connectivityMatrixViewController,
                              "Connectivity");
     }
+    if (isFeaturesToolBox) {
+        m_annotationViewController = new AnnotationSelectionViewController(browserWindowIndex,
+                                                                           this);
+        m_annotationTabIndex = addToTabWidget(m_annotationViewController,
+                                              "Annot");
+    }
     if (isFeaturesToolBox) {
         m_borderSelectionViewController = new BorderSelectionViewController(browserWindowIndex,
                                                                                 this);
@@ -330,6 +351,10 @@ BrainBrowserWindowOrientedToolBox::saveToScene(const SceneAttributes* sceneAttri
     /*
      * Save controllers in the toolbox
      */
+    if (m_annotationViewController != NULL) {
+        sceneClass->addClass(m_annotationViewController->saveToScene(sceneAttributes,
+                                                                     "m_annotationViewController"));
+    }
     if (m_borderSelectionViewController != NULL) {
         sceneClass->addClass(m_borderSelectionViewController->saveToScene(sceneAttributes,
                                                      "m_borderSelectionViewController"));
@@ -401,6 +426,10 @@ BrainBrowserWindowOrientedToolBox::restoreFromScene(const SceneAttributes* scene
     /*
      * Restore controllers in the toolbox
      */
+    if (m_annotationViewController != NULL) {
+        m_annotationViewController->restoreFromScene(sceneAttributes,
+                                                     sceneClass->getClass("m_annotationViewController"));
+    }
     if (m_borderSelectionViewController != NULL) {
         m_borderSelectionViewController->restoreFromScene(sceneAttributes,
                                                           sceneClass->getClass("m_borderSelectionViewController"));
@@ -444,7 +473,15 @@ BrainBrowserWindowOrientedToolBox::restoreFromScene(const SceneAttributes* scene
                              sceneClass->getClass("geometry"));
     }
     else {
+        const SceneClass* geometryClass = sceneClass->getClass("geometry");
+        if (geometryClass != NULL) {
+            /* is restored only when floating and visible. */
+            geometryClass->setDescendantsRestored(true);
+        }
+#if QT_VERSION < 0x050000
         /*
+         * Do this for Qt4 only.  Qt5 restores size in BrainBrowserWindow.
+         *
          * From http://stackoverflow.com/questions/2722939/c-resize-a-docked-qt-qdockwidget-programmatically
          *
          * Set the minimum and maximum sizes and restore them later.
@@ -469,6 +506,7 @@ BrainBrowserWindowOrientedToolBox::restoreFromScene(const SceneAttributes* scene
                                this,
                                SLOT(restoreMinimumAndMaximumSizesAfterSceneRestored()));
         }
+#endif
     }
 }
 
@@ -504,6 +542,7 @@ BrainBrowserWindowOrientedToolBox::receiveEvent(Event* event)
         /*
          * Determine types of data this is loaded
          */
+        bool haveAnnotation = ( ! brain->getSceneAnnotationFile()->isEmpty());
         bool haveBorders    = false;
         bool haveConnFiles  = false;
         bool haveFibers     = false;
@@ -522,12 +561,18 @@ BrainBrowserWindowOrientedToolBox::receiveEvent(Event* event)
             
             const DataFileTypeEnum::Enum dataFileType = caretDataFile->getDataFileType();
             switch (dataFileType) {
+                case DataFileTypeEnum::ANNOTATION:
+                    haveAnnotation = true;
+                    break;
                 case DataFileTypeEnum::BORDER:
                     haveBorders = true;
                     break;
                 case DataFileTypeEnum::CONNECTIVITY_DENSE:
                     haveConnFiles = true;
                     break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC:
+                    haveConnFiles = true;
+                    break;
                 case DataFileTypeEnum::CONNECTIVITY_DENSE_LABEL:
                     haveLabels = true;
                     break;
@@ -605,8 +650,9 @@ BrainBrowserWindowOrientedToolBox::receiveEvent(Event* event)
         int defaultTabIndex = -1;
         bool enableLayers = true;
         bool enableVolumeSurfaceOutline = false;
-        bool enableCharts = false;
-        EventBrowserWindowContentGet browserContentEvent(m_browserWindowIndex);
+        bool enableChartOne = false;
+        bool enableChartTwo = false;
+        EventBrowserWindowDrawingContent browserContentEvent(m_browserWindowIndex);
         EventManager::get()->sendEvent(browserContentEvent.getPointer());
         BrowserTabContent* windowContent = browserContentEvent.getSelectedBrowserTabContent();
             if (windowContent != NULL) {
@@ -631,13 +677,23 @@ BrainBrowserWindowOrientedToolBox::receiveEvent(Event* event)
                         break;
                     case ModelTypeEnum::MODEL_TYPE_CHART:
                         defaultTabIndex = m_chartTabIndex;
+                        enableChartOne = true;
+                        enableLayers = false;
+                        enableVolumeSurfaceOutline = false;
+                        haveBorders = false;
+                        haveFibers  = false;
+                        haveFoci    = false;
+                        haveLabels  = false;
+                        break;
+                    case ModelTypeEnum::MODEL_TYPE_CHART_TWO:
+                        defaultTabIndex = m_chartOverlayTabIndex;
+                        enableChartTwo = true;
                         enableLayers = false;
                         enableVolumeSurfaceOutline = false;
                         haveBorders = false;
                         haveFibers  = false;
                         haveFoci    = false;
                         haveLabels  = false;
-                        enableCharts = true;
                         break;
                 }
             }
@@ -655,10 +711,12 @@ BrainBrowserWindowOrientedToolBox::receiveEvent(Event* event)
          * NOTE: Order is important so that overlay tab is 
          * automatically selected.
          */
-        if (m_chartTabIndex >= 0) m_tabWidget->setTabEnabled(m_chartTabIndex, enableCharts);
+        if (m_chartTabIndex >= 0) m_tabWidget->setTabEnabled(m_chartTabIndex, enableChartOne);
+        if (m_chartOverlayTabIndex >= 0) m_tabWidget->setTabEnabled(m_chartOverlayTabIndex, enableChartTwo);
         if (m_connectivityTabIndex >= 0) m_tabWidget->setTabEnabled(m_connectivityTabIndex, haveConnFiles);
         if (m_volumeSurfaceOutlineTabIndex >= 0) m_tabWidget->setTabEnabled(m_volumeSurfaceOutlineTabIndex, enableVolumeSurfaceOutline);
         
+        if (m_annotationTabIndex >= 0) m_tabWidget->setTabEnabled(m_annotationTabIndex, haveAnnotation);
         if (m_borderTabIndex >= 0) m_tabWidget->setTabEnabled(m_borderTabIndex, haveBorders);
         if (m_fiberOrientationTabIndex >= 0) m_tabWidget->setTabEnabled(m_fiberOrientationTabIndex, haveFibers);
         if (m_fociTabIndex >= 0) m_tabWidget->setTabEnabled(m_fociTabIndex, haveFoci);
