@@ -32,8 +32,10 @@
 #undef __OVERLAY_SETTINGS_EDITOR_DIALOG_DECLARE__
 
 #include "CaretMappableDataFile.h"
+#include "ChartTwoOverlay.h"
 #include "CiftiFiberTrajectoryFile.h"
 #include "CiftiConnectivityMatrixParcelFile.h"
+#include "EventChartOverlayValidate.h"
 #include "EventDataFileDelete.h"
 #include "EventGraphicsUpdateAllWindows.h"
 #include "EventManager.h"
@@ -41,6 +43,8 @@
 #include "EventSurfaceColoringInvalidate.h"
 #include "EventUserInterfaceUpdate.h"
 #include "GiftiLabelTableEditor.h"
+#include "MapSettingsChartTwoLineHistoryWidget.h"
+#include "MapSettingsColorBarWidget.h"
 #include "MapSettingsFiberTrajectoryWidget.h"
 #include "MapSettingsLabelsWidget.h"
 #include "MapSettingsLayerWidget.h"
@@ -76,7 +80,10 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     this->setDeleteWhenClosed(false);
 
     m_caretMappableDataFile = NULL;
-    m_mapIndex = -1;
+    m_selectedMapFileIndex = -1;
+    m_brainordinateOverlay = NULL;
+    m_chartOverlay = NULL;
+    m_chartOverlaySelectedIndexType = ChartTwoOverlay::SelectedIndexType::INVALID;
     
     /*
      * No apply button
@@ -85,6 +92,8 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     
     QWidget* mapNameWidget = createMapFileAndNameSection();
     
+    m_colorBarWidget = new MapSettingsColorBarWidget();
+    
     m_fiberTrajectoryWidget = new MapSettingsFiberTrajectoryWidget();
     
     m_layerWidget = new MapSettingsLayerWidget();
@@ -97,7 +106,13 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     
     m_labelsWidget = new MapSettingsLabelsWidget();
     
+    m_lineHistoryWidget = new MapSettingsChartTwoLineHistoryWidget();
+    
     m_tabWidget = new QTabWidget();
+    
+    m_colorBarWidgetTabIndex = m_tabWidget->addTab(m_colorBarWidget,
+                                                   "Color Bar");
+    
     m_labelsWidgetTabIndex = m_tabWidget->addTab(m_labelsWidget,
                       "Labels");
     m_tabWidget->setTabEnabled(m_tabWidget->count() - 1, false);
@@ -105,6 +120,8 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     m_layersWidgetTabIndex = m_tabWidget->addTab(m_layerWidget,
                       "Layer");
     
+    m_lineHistoryWidgetTabIndex = m_tabWidget->addTab(m_lineHistoryWidget,
+                                                      "Lines");
     m_metadataWidgetTabIndex = m_tabWidget->addTab(new QWidget(),
                       "Metadata");
     m_tabWidget->setTabEnabled(m_tabWidget->count() - 1, false);
@@ -194,44 +211,94 @@ OverlaySettingsEditorDialog::createMapFileAndNameSection()
  *     The focus event.
  */
 void
-OverlaySettingsEditorDialog::focusInEvent(QFocusEvent* /*event*/)
+OverlaySettingsEditorDialog::focusInEvent(QFocusEvent* event)
 {
+    if (event->reason() == Qt::PopupFocusReason) {
+        /*
+         * This occurs when a combo box is popped up Mac,
+         * causes a problem, and can be ignored.
+         */
+        return;
+    }
+    
     updateDialog();
 }
 
 /**
  * Update if the given overlay is displayed in the dialog.
  * 
- * @param overlay
- *    The overlay.
+ * @param brainordinateOverlay
+ *    Brainordinate overlay for the dialog.
+ * @param chartOverlay
+ *    Chart overlay for the dialog.
  */
 void
-OverlaySettingsEditorDialog::updateIfThisOverlayIsInDialog(Overlay* overlay)
+OverlaySettingsEditorDialog::updateIfThisOverlayIsInDialog(Overlay* brainordinateOverlay,
+                                                           ChartTwoOverlay* chartOverlay)
 {
-    if (m_overlay == overlay) {
-        updateDialogContent(m_overlay);
+    if (m_brainordinateOverlay != NULL) {
+        if (m_brainordinateOverlay == brainordinateOverlay) {
+            updateDialogContent(m_brainordinateOverlay,
+                                NULL);
+        }
+    }
+    else if (m_chartOverlay != NULL) {
+        if (m_chartOverlay == chartOverlay) {
+            updateDialogContent(NULL,
+                                m_chartOverlay);
+        }
     }
 }
 
 /**
  * May be called to update the dialog's content.
  *
- * @param overlay
- *    Overlay for the dialog.
+ * @param brainordinateOverlay
+ *    Brainordinate overlay for the dialog.
+ * @param chartOverlay
+ *    Chart overlay for the dialog.
  */
 void 
-OverlaySettingsEditorDialog::updateDialogContent(Overlay* overlay)
+OverlaySettingsEditorDialog::updateDialogContent(Overlay* brainordinateOverlay,
+                                                 ChartTwoOverlay* chartOverlay)
 {
+    updateDialogContentPrivate(brainordinateOverlay,
+                               chartOverlay);
+}
+
+/**
+ * Update the dialog's content.  Only one of the parameters
+ * should be non-NULL.
+ *
+ * @param brainordinateOverlay
+ *    Brainordinate overlay for the dialog.
+ * @param chartOverlay
+ *    Chart overlay for the dialog.
+ */
+void
+OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOverlay,
+                                                        ChartTwoOverlay* chartOverlay)
+{
+    if ((brainordinateOverlay != NULL)
+        && (chartOverlay != NULL)) {
+        CaretAssert(0);  // only one should be non-NULL
+    }
+    
     const int32_t selectedTabIndex = m_tabWidget->currentIndex();
-    m_overlay = overlay;
-//    m_caretMappableDataFile = caretMappableDataFile;
-//    m_mapFileIndex = mapFileIndex;
+    m_brainordinateOverlay = brainordinateOverlay;
+    m_chartOverlay = chartOverlay;
     
     m_caretMappableDataFile = NULL;
-    m_mapIndex = -1;
-    if (m_overlay != NULL) {
-        overlay->getSelectionData(m_caretMappableDataFile,
-                                  m_mapIndex);
+    m_chartOverlaySelectedIndexType = ChartTwoOverlay::SelectedIndexType::INVALID;
+    m_selectedMapFileIndex = -1;
+    if (m_brainordinateOverlay != NULL) {
+        m_brainordinateOverlay->getSelectionData(m_caretMappableDataFile,
+                                                 m_selectedMapFileIndex);
+    }
+    if (m_chartOverlay != NULL) {
+        m_chartOverlay->getSelectionData(m_caretMappableDataFile,
+                                         m_chartOverlaySelectedIndexType,
+                                         m_selectedMapFileIndex);
     }
     
     bool isLabelsValid = false;
@@ -241,92 +308,210 @@ OverlaySettingsEditorDialog::updateDialogContent(Overlay* overlay)
     bool isParcelsValid = false;
     bool isFiberTrajectoryValid = false;
     bool isVolumeLayer = false;
+    bool isLinesValid = false;
     
     QString mapFileName = "";
     QString mapName = "";
     
     if (m_caretMappableDataFile != NULL) {
-        if ((m_mapIndex >= 0)
-            && (m_mapIndex < m_caretMappableDataFile->getNumberOfMaps())) {
-            /*
-             * Get name of file and map
-             */
-            mapFileName = m_caretMappableDataFile->getFileNameNoPath();
-            if (m_mapIndex >= 0) {
-                mapName = m_caretMappableDataFile->getMapName(m_mapIndex);
-            }
-            
-            /*
-             * Update layer widget
-             */
-            m_layerWidget->updateContent(m_overlay);
-            
-            if (m_caretMappableDataFile->isMappedWithLabelTable()) {
-                if (m_caretMappableDataFile->getMapLabelTable(m_mapIndex) != NULL) {
+        if (m_brainordinateOverlay != NULL) {
+            if ((m_selectedMapFileIndex >= 0)
+                && (m_selectedMapFileIndex < m_caretMappableDataFile->getNumberOfMaps())) {
+                /*
+                 * Get name of file and map
+                 */
+                mapFileName = m_caretMappableDataFile->getFileNameNoPath();
+                if (m_selectedMapFileIndex >= 0) {
+                    mapName = m_caretMappableDataFile->getMapName(m_selectedMapFileIndex);
+                }
+                
+                /*
+                 * Update layer widget
+                 */
+                m_layerWidget->updateContent(m_brainordinateOverlay);
+                
+                if (m_caretMappableDataFile->isMappedWithLabelTable()) {
+                    if (m_caretMappableDataFile->getMapLabelTable(m_selectedMapFileIndex) != NULL) {
+                        /*
+                         * Update label editor
+                         */
+                        isLabelsValid = true;
+                        m_labelsWidget->updateContent(m_brainordinateOverlay);
+                    }
+                }
+                
+                metadata = m_caretMappableDataFile->getMapMetaData(m_selectedMapFileIndex);
+                if (metadata != NULL) {
                     /*
-                     * Update label editor
+                     * TODO: Update metadata widget
                      */
-                    isLabelsValid = true;
-                    m_labelsWidget->updateContent(overlay);
+                    //isMetadataValid = true;
                 }
-            }
-            
-            metadata = m_caretMappableDataFile->getMapMetaData(m_mapIndex);
-            if (metadata != NULL) {
-                /*
-                 * TODO: Update metadata widget
-                 */
-                //isMetadataValid = true;
-            }
-            
-            if (m_caretMappableDataFile->isMappedWithPalette()) {
-                if (m_caretMappableDataFile->getMapPaletteColorMapping(m_mapIndex) != NULL) {
+                
+                if (m_caretMappableDataFile->isMappedWithPalette()) {
+                    PaletteColorMapping* paletteColorMapping = m_caretMappableDataFile->getMapPaletteColorMapping(m_selectedMapFileIndex);
+                    if (paletteColorMapping != NULL) {
+                        /*
+                         * Update palette settings
+                         */
+                        isPaletteValid = true;
+                        m_paletteColorMappingWidget->updateEditor(m_caretMappableDataFile,
+                                                                  m_selectedMapFileIndex);
+                        AnnotationColorBar* colorBar = NULL;
+                        if (m_brainordinateOverlay != NULL) {
+                            colorBar = m_brainordinateOverlay->getColorBar();
+                        }
+                        else if (m_chartOverlay != NULL) {
+                            colorBar = m_chartOverlay->getColorBar();
+                        }
+                        m_colorBarWidget->updateContent(m_caretMappableDataFile,
+                                                        m_selectedMapFileIndex,
+                                                        colorBar,
+                                                        paletteColorMapping);
+                    }
+                }
+                
+                CiftiFiberTrajectoryFile* trajFile = dynamic_cast<CiftiFiberTrajectoryFile*>(m_caretMappableDataFile);
+                if (trajFile != NULL) {
                     /*
-                     * Update palette settings
+                     * Update trajectory
                      */
-                    isPaletteValid = true;
-                    m_paletteColorMappingWidget->updateEditor(m_caretMappableDataFile,
-                                                              m_mapIndex);
+                    isFiberTrajectoryValid = true;
+                    m_fiberTrajectoryWidget->updateEditor(trajFile);
                 }
-            }
-            
-            CiftiFiberTrajectoryFile* trajFile = dynamic_cast<CiftiFiberTrajectoryFile*>(m_caretMappableDataFile);
-            if (trajFile != NULL) {
+                
+                CiftiConnectivityMatrixParcelFile* parcelsFile = dynamic_cast<CiftiConnectivityMatrixParcelFile*>(m_caretMappableDataFile);
+                if (parcelsFile != NULL) {
+                    /*
+                     * Update parcels
+                     */
+                    isParcelsValid = true;
+                    m_parcelsWidget->updateEditor(parcelsFile);
+                }
+                
                 /*
-                 * Update trajectory
+                 * Is volume mappable
                  */
-                isFiberTrajectoryValid = true;
-                m_fiberTrajectoryWidget->updateEditor(trajFile);
-            }
-
-            CiftiConnectivityMatrixParcelFile* parcelsFile = dynamic_cast<CiftiConnectivityMatrixParcelFile*>(m_caretMappableDataFile);
-            if (parcelsFile != NULL) {
-                /*
-                 * Update parcels
-                 */
-                isParcelsValid = true;
-                m_parcelsWidget->updateEditor(parcelsFile);
-            }
-            
-            /*
-             * Is volume mappable
-             */
-            if (m_caretMappableDataFile->isVolumeMappable()) {
-                if (isFiberTrajectoryValid) {
-                    /* nothing */
-                }
-                else {
-                    isVolumeLayer = true;
+                if (m_caretMappableDataFile->isVolumeMappable()) {
+                    if (isFiberTrajectoryValid) {
+                        /* nothing */
+                    }
+                    else {
+                        isVolumeLayer = true;
+                    }
                 }
             }
+        }
+        else if (m_chartOverlay != NULL) {
+            bool hasMapsFlag = false;
+            bool hasHistoryFlag = false;
+            switch (m_chartOverlay->getChartTwoDataType()) {
+                case ChartTwoDataTypeEnum::CHART_DATA_TYPE_INVALID:
+                    break;
+                case ChartTwoDataTypeEnum::CHART_DATA_TYPE_HISTOGRAM:
+                    hasMapsFlag = true;
+                    break;
+                case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_SERIES:
+                    hasHistoryFlag = true;
+                    break;
+                case ChartTwoDataTypeEnum::CHART_DATA_TYPE_MATRIX:
+                    hasMapsFlag = true;
+                    break;
+            }
             
+            int32_t selectedMapIndex = -1;
+            bool mapTypeFlag = false;
+            if (hasMapsFlag) {
+                switch (m_chartOverlaySelectedIndexType) {
+                    case ChartTwoOverlay::SelectedIndexType::INVALID:
+                        CaretAssert(0);
+                        break;
+                    case ChartTwoOverlay::SelectedIndexType::MAP:
+                        if ((m_selectedMapFileIndex >= 0)
+                            && (m_selectedMapFileIndex < m_caretMappableDataFile->getNumberOfMaps())) {
+                            selectedMapIndex = m_selectedMapFileIndex;
+                            mapTypeFlag = true;
+                        }
+                        break;
+                    case ChartTwoOverlay::SelectedIndexType::COLUMN:
+                    case ChartTwoOverlay::SelectedIndexType::ROW:
+                        if (m_selectedMapFileIndex >= 0) {
+                            if (m_caretMappableDataFile->getNumberOfMaps() == 1) {
+                                m_selectedMapFileIndex = 0;
+                            }
+                            selectedMapIndex = m_selectedMapFileIndex;
+                        }
+                        break;
+                }
+                if ((selectedMapIndex >= 0)
+                    && (selectedMapIndex < m_caretMappableDataFile->getNumberOfMaps())) {
+                    /*
+                     * Get name of file and map
+                     */
+                    mapFileName = m_caretMappableDataFile->getFileNameNoPath();
+                    if (m_selectedMapFileIndex >= 0) {
+                        mapName = m_caretMappableDataFile->getMapName(selectedMapIndex);
+                    }
+                    
+                    if (mapTypeFlag) {
+                        metadata = m_caretMappableDataFile->getMapMetaData(selectedMapIndex);
+                        if (metadata != NULL) {
+                            /*
+                             * TODO: Update metadata widget
+                             */
+                            //isMetadataValid = true;
+                        }
+                    }
+                    
+                    if (m_caretMappableDataFile->isMappedWithPalette()) {
+                        PaletteColorMapping* paletteColorMapping = m_caretMappableDataFile->getMapPaletteColorMapping(selectedMapIndex);
+                        if (paletteColorMapping != NULL) {
+                            /*
+                             * Update palette settings
+                             */
+                            isPaletteValid = true;
+                            m_paletteColorMappingWidget->updateEditor(m_caretMappableDataFile,
+                                                                      selectedMapIndex);
+                            AnnotationColorBar* colorBar = NULL;
+                            if (m_brainordinateOverlay != NULL) {
+                                colorBar = m_brainordinateOverlay->getColorBar();
+                            }
+                            else if (m_chartOverlay != NULL) {
+                                colorBar = m_chartOverlay->getColorBar();
+                            }
+                            m_colorBarWidget->updateContent(m_caretMappableDataFile,
+                                                            selectedMapIndex,
+                                                            colorBar,
+                                                            paletteColorMapping);
+                        }
+                    }
+                    
+                    CiftiConnectivityMatrixParcelFile* parcelsFile = dynamic_cast<CiftiConnectivityMatrixParcelFile*>(m_caretMappableDataFile);
+                    if (parcelsFile != NULL) {
+                        /*
+                         * Update parcels
+                         */
+                        isParcelsValid = true;
+                        m_parcelsWidget->updateEditor(parcelsFile);
+                    }
+                }
+            }
+            else if (hasHistoryFlag) {
+                isLinesValid = true;
+                m_lineHistoryWidget->updateContent(m_chartOverlay);
+            }
+        }
+        else {
+            CaretAssert(0);
         }
     }
-  
+
   
     /*
      * Set enabled status of tabs
      */
+    m_tabWidget->setTabEnabled(m_colorBarWidgetTabIndex,
+                               isPaletteValid);
     m_tabWidget->setTabEnabled(m_labelsWidgetTabIndex,
                                isLabelsValid);
     m_tabWidget->setTabEnabled(m_layersWidgetTabIndex,
@@ -339,6 +524,8 @@ OverlaySettingsEditorDialog::updateDialogContent(Overlay* overlay)
                                isParcelsValid);
     m_tabWidget->setTabEnabled(m_trajectoryWidgetTabIndex,
                                isFiberTrajectoryValid);
+    m_tabWidget->setTabEnabled(m_lineHistoryWidgetTabIndex,
+                               isLinesValid);
 
     /*
      * When the selected tab is invalid, we want to select the
@@ -347,6 +534,8 @@ OverlaySettingsEditorDialog::updateDialogContent(Overlay* overlay)
      */
     std::vector<int32_t> priorityTabIndices;
     priorityTabIndices.push_back(m_paletteWidgetTabIndex);
+    priorityTabIndices.push_back(m_lineHistoryWidgetTabIndex);
+    priorityTabIndices.push_back(m_colorBarWidgetTabIndex);
     priorityTabIndices.push_back(m_labelsWidgetTabIndex);
     priorityTabIndices.push_back(m_parcelsWidgetTabIndex);
     priorityTabIndices.push_back(m_trajectoryWidgetTabIndex);
@@ -386,19 +575,52 @@ OverlaySettingsEditorDialog::updateDialog()
     /*
      * Validate overlay to prevent crash
      */
-    EventOverlayValidate validateOverlayEvent(m_overlay);
-    EventManager::get()->sendEvent(validateOverlayEvent.getPointer());
-    if (validateOverlayEvent.isValidOverlay() == false) {
-        m_overlay = NULL;
+    
+    if (m_brainordinateOverlay != NULL) {
+        EventOverlayValidate validateOverlayEvent(m_brainordinateOverlay);
+        EventManager::get()->sendEvent(validateOverlayEvent.getPointer());
+        if ( ! validateOverlayEvent.isValidOverlay()) {
+            m_brainordinateOverlay = NULL;
+        }
     }
     
-    updateDialogContent(m_overlay);
+    if (m_chartOverlay != NULL) {
+        EventChartOverlayValidate validateChartOverlayEvent(m_chartOverlay);
+        EventManager::get()->sendEvent(validateChartOverlayEvent.getPointer());
+        if ( ! validateChartOverlayEvent.isValidChartOverlay()) {
+            m_chartOverlay = NULL;
+        }
+    }
+
+    updateDialogContentPrivate(m_brainordinateOverlay,
+                               m_chartOverlay);
     
-    if (m_overlay == NULL) {
+    if ((m_brainordinateOverlay == NULL)
+        && (m_chartOverlay == NULL)) {
         close();
     }
 }
 
+/**
+ * Update chart lines in the dialog
+ */
+void
+OverlaySettingsEditorDialog::updateChartLinesInDialog()
+{
+    if (m_chartOverlay != NULL) {
+        EventChartOverlayValidate validateChartOverlayEvent(m_chartOverlay);
+        EventManager::get()->sendEvent(validateChartOverlayEvent.getPointer());
+        if ( ! validateChartOverlayEvent.isValidChartOverlay()) {
+            m_chartOverlay = NULL;
+        }
+        
+        if (m_chartOverlay != NULL) {
+            updateDialogContentPrivate(NULL,
+                                       m_chartOverlay);
+        }
+    }
+}
+
 
 /**
  * Called when close button pressed.
@@ -411,7 +633,8 @@ OverlaySettingsEditorDialog::closeButtonPressed()
      * and we don't want the dialog updating if it is not
      * visible.
      */
-    updateDialogContent(NULL);
+    updateDialogContent(NULL,
+                        NULL);
     
     /*
      * Allow this dialog to be reused (checked means DO NOT reuse)
@@ -489,9 +712,9 @@ OverlaySettingsEditorDialog::editLabelTablePushButtonClicked()
 {
     if (m_caretMappableDataFile != NULL) {
         if (m_caretMappableDataFile->isMappedWithLabelTable()) {
-            if (m_mapIndex >= 0) {
+            if (m_selectedMapFileIndex >= 0) {
                 GiftiLabelTableEditor labelTableEditor(m_caretMappableDataFile,
-                                                       m_mapIndex,
+                                                       m_selectedMapFileIndex,
                                                        "Edit Labels",
                                                        GiftiLabelTableEditor::OPTION_ADD_APPLY_BUTTON,
                                                        m_editLabelTablePushButton);
