@@ -21,17 +21,28 @@
  */
 /*LICENSE_END*/
 
+#include <set>
+
 #include <AString.h>
 
 #include "CaretObject.h"
 
+#include "AnnotationColorBarNumericText.h"
+#include "CaretColorEnum.h"
+#include "NumericFormatModeEnum.h"
+#include "PaletteColorBarValuesModeEnum.h"
 #include "PaletteEnums.h"
+#include "PaletteHistogramRangeModeEnum.h"
+#include "PaletteInvertModeEnum.h"
+#include "PaletteModifiedStatusEnum.h"
 #include "PaletteThresholdRangeModeEnum.h"
 #include "XmlException.h"
 
 namespace caret {
 
+    class AnnotationColorBar;
     class FastStatistics;
+    class Palette;
     class XmlWriter;
     
     /**
@@ -39,7 +50,7 @@ namespace caret {
      */
     class PaletteColorMapping : public CaretObject {
         
-    public:
+    public:        
         PaletteColorMapping();
         
         PaletteColorMapping(const PaletteColorMapping& o);
@@ -52,23 +63,27 @@ namespace caret {
         
         virtual ~PaletteColorMapping();
         
-        void copy(const PaletteColorMapping& pcm);
+        void copy(const PaletteColorMapping& pcm,
+                  const bool copyHistogramAttributesFlag);
         
     private:
-        void copyHelper(const PaletteColorMapping& o);
+        void copyHelper(const PaletteColorMapping& o,
+                        const bool copyHistogramAttributesFlag);
         
         void initializeMembersPaletteColorMapping();
         
     public:
-        void writeAsXML(XmlWriter& xmlWriter)
-       ;
+        void setupAnnotationColorBar(const FastStatistics* statistics,
+                                     AnnotationColorBar* colorBar);
+
+        void setupAnnotationColorBarNumericText(const FastStatistics* statistics,
+                                                AnnotationColorBar* colorBar);
         
-        AString encodeInXML()
-       ;
+        void writeAsXML(XmlWriter& xmlWriter);
         
+        AString encodeInXML();
         
-        void decodeFromStringXML(const AString& xml)
-       ;
+        void decodeFromStringXML(const AString& xml);
         
         float getAutoScalePercentageNegativeMaximum() const;
         
@@ -110,6 +125,10 @@ namespace caret {
         
         void setInterpolatePaletteFlag(const bool interpolatePaletteFlag);
         
+        PaletteInvertModeEnum::Enum getInvertedMode() const;
+        
+        void setInvertedMode(const PaletteInvertModeEnum::Enum invertedMode);
+        
         PaletteScaleModeEnum::Enum getScaleMode() const;
         
         void setScaleMode(const PaletteScaleModeEnum::Enum scaleMode);
@@ -118,6 +137,8 @@ namespace caret {
         
         void setSelectedPaletteName(const AString& selectedPaletteName);
         
+        const Palette* getPalette() const;
+        
         void setSelectedPaletteToPsych();
         
         void setSelectedPaletteToPsychNoNone();
@@ -200,21 +221,71 @@ namespace caret {
         
         void setThresholdNegMinPosMaxLinked(const bool linked);
         
+        PaletteHistogramRangeModeEnum::Enum getHistogramRangeMode() const;
+        
+        void setHistogramRangeMode(const PaletteHistogramRangeModeEnum::Enum histogramRangeMode);
+        
+        bool isHistogramBarsVisible() const;
+        
+        void setHistogramBarsVisible(const bool histogramBarsVisible);
+        
+        bool isHistogramEnvelopeVisible() const;
+        
+        void setHistogramEnvelopeVisible(const bool histogramEnvelopeVisible);
+        
+        CaretColorEnum::Enum getHistogramBarsColor() const;
+        
+        void setHistogramBarsColor(const CaretColorEnum::Enum histogramBarsColor);
+        
+        CaretColorEnum::Enum getHistogramEnvelopeColor() const;
+        
+        void setHistogramEnvelopeColor(const CaretColorEnum::Enum histogramEnvelopeColor);
+        
+        float getHistogramEnvelopeLineWidthPercentage() const;
+        
+        void setHistogramEnvelopeLineWidthPercentage(const float lineWidthPercentage);
+        
+        int32_t getHistogramNumberOfBuckets() const;
+        
+        void setHistogramNumberOfBuckets(const int32_t histogramNumberOfBuckets);
+    
+        NumericFormatModeEnum::Enum getColorBarNumericFormatMode() const;
+        
+        int32_t getColorBarPrecisionDigits() const;
+        
+        int32_t getColorBarNumericSubdivisionCount() const;
+        
+        void setColorBarNumericFormatMode(const NumericFormatModeEnum::Enum colorBarNumericFormatMode);
+        
+        void setColorBarPrecisionDigits(const int32_t colorBarPrecisionDigits);
+        
+        void setColorBarNumericSubdivisionCount(const int32_t colroBarNumericSubdivisionCount);
+        
+        PaletteColorBarValuesModeEnum::Enum getColorBarValuesMode() const;
+        
+        void setColorBarValuesMode(const PaletteColorBarValuesModeEnum::Enum colorBarValuesMode);
+        
+        bool isColorBarShowTickMarksSelected() const;
+        
+        void setColorBarShowTickMarksSelected(const bool selected);
+        
         void setModified();
         
         void clearModified();
         
         bool isModified() const;
         
+        void setSceneModified();
+        
+        PaletteModifiedStatusEnum::Enum getModifiedStatus() const;
+        
         void mapDataToPaletteNormalizedValues(const FastStatistics* statistics,
                                               const float* dataValues,
                                               float* normalizedValuesOut,
                                               const int64_t numberOfData) const;
         
         void getPaletteColorBarScaleText(const FastStatistics* statistics,
-                                         AString& minimumValueTextOut,
-                                         AString& zeroValueTextOut,
-                                         AString& maximumValueTextOut) const;
+                                         std::vector<AnnotationColorBarNumericText*>& colorBarNumericTextOut) const;
         
         /** A positive value near zero - may be zero! */
         static const float SMALL_POSITIVE;
@@ -222,7 +293,12 @@ namespace caret {
         /** A negative value near zero - may be zero! */
         static const float SMALL_NEGATIVE;
         
+        /** TSC: the excluded zone of normalization is a separate issue to zero detection in the data
+          * specifically, it is a HACK, in order for palettes to be able to specify a special color for data that is 0, which is not involved in color interpolation */
+        static const float PALETTE_ZERO_COLOR_ZONE;
     private:
+        bool isZeroNumericText(const AString& numericText) const;
+        
         PaletteScaleModeEnum::Enum scaleMode;
         
         float autoScalePercentageNegativeMaximum;
@@ -249,6 +325,8 @@ namespace caret {
         
         bool interpolatePaletteFlag;
         
+        PaletteInvertModeEnum::Enum invertedMode = PaletteInvertModeEnum::OFF;
+        
         bool displayPositiveDataFlag;
         
         bool displayZeroDataFlag;
@@ -279,14 +357,42 @@ namespace caret {
         
         bool thresholdNegMinPosMaxLinked;
         
-        /**Tracks modification, DO NOT copy */
-        bool modifiedFlag;
+        PaletteHistogramRangeModeEnum::Enum histogramRangeMode;
         
+        bool histogramBarsVisible;
+
+        bool histogramEnvelopeVisible;
+        
+        CaretColorEnum::Enum histogramBarsColor;
+        
+        CaretColorEnum::Enum histogramEnvelopeColor;
+        
+        float histogramEnvelopeLineWidthPercentage;
+        
+        int32_t histogramNumberOfBuckets;
+        
+        NumericFormatModeEnum::Enum colorBarNumericFormatMode;
+        
+        int32_t colorBarPrecisionDigits;
+        
+        int32_t colorBarNumericSubdivisionCount;
+        
+        PaletteColorBarValuesModeEnum::Enum colorBarValuesMode;
+    
+        bool colorBarShowTickMarksSelected;
+        
+        /**Tracks modification, DO NOT copy */
+        PaletteModifiedStatusEnum::Enum modifiedStatus;
+        
+        /** keeps missing palettes from being logged more than once */
+        static std::set<AString> s_missingPaletteNames;
     };
 
 #ifdef __PALETTE_COLOR_MAPPING_DECLARE__
-    const float PaletteColorMapping::SMALL_POSITIVE = 0.0;  // JWH 24 April 2015  0.00001;
-    const float PaletteColorMapping::SMALL_NEGATIVE = 0.0;  // JWH 24 April 2015 -0.00001;
+    const float PaletteColorMapping::SMALL_POSITIVE = 0.0;
+    const float PaletteColorMapping::SMALL_NEGATIVE = 0.0;
+    const float PaletteColorMapping::PALETTE_ZERO_COLOR_ZONE = 0.00001f;
+    std::set<AString> PaletteColorMapping::s_missingPaletteNames;
 #endif // __PALETTE_COLOR_MAPPING_DECLARE__
     
 } // namespace
